from django.forms import BoundField, CharField


class PrefixedBoundField(BoundField):
    """
    A bound field that passes the form's prefix into the widget's attrs
    """
    def as_widget(self, widget=None, attrs=None, only_initial=False):
        attrs = {} if attrs is None else attrs.copy()
        if self.form.prefix:
            attrs.update({
                'prefix': self.form.prefix,
            })
        return super(PrefixedBoundField, self).as_widget(widget, attrs, only_initial)


class PrefixedFormFieldMixin(object):
    """
    A form field that binds to a custom bound field class, so we can pass the
    form's prefix into the widget's attrs
    """
    def get_bound_field(self, form, field_name):
        """
        For Django 1.9+

        Return a BoundField instance that will be used when accessing the form
        field in a template.
        """
        return PrefixedBoundField(form, self, field_name)


class OSMFormField(PrefixedFormFieldMixin, CharField):
    pass



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.core import checks
from django.core.exceptions import FieldDoesNotExist
from django.db.models.fields import FloatField, TextField
from django.utils.encoding import force_text

from .forms import OSMFormField
from .validators import validate_latitude, validate_longitude
from .widgets import OSMWidget


class Location(object):
    """
    A wrapper class bundling the description of a location (``text``) and its
    geo coordinates, latitude (``lat``) and longitude (``lon``).

    :param float lat: The latitude
    :param float lon: The longitude
    :param str: The description
    """

    def __init__(self, lat, lon, text):
        self.lat = lat
        self.lon = lon
        self.text = text

    def __str__(self):
        """
        Returns a string representation of this object in the form ``text (lat,
        lon)`` where either ``text`` or ``(lat, lon)'`` or both can be empty. In
        that case the return value is ``''``.
        """
        out = []
        if self.text is not None:
            out.append(self.text)
        if self.lat is not None and self.lon is not None:
            out.append('(%.6f, %.6f)' % (self.lat, self.lon))
        return ' '.join(out)

    def __repr__(self):
        return '<Location lat=%.6f lon=%.6f text=%s>' % (
            self.lat, self.lon, force_text(self.text)
        )

    def __copy__(self):
        return self.__class__(self.lat, self.lon, self.text)

    def __eq__(self, other):
        return (
            isinstance(other, self.__class__)
            and self.lat == other.lat
            and self.lon == other.lon
            and self.text == other.text
        )

    def __ne__(self, other):
        return not (self == other)


class LatitudeField(FloatField):
    """
    Bases: :class:`django.db.models.FloatField`

    All :ref:`default field options <django:common-model-field-options>`.

    The ``validators`` parameter will be appended with
    :func:`~validate_latitude` if not already present.
    """

    def __init__(self, *args, **kwargs):
        kwargs.setdefault('validators', [])
        if validate_latitude not in kwargs['validators']:
            kwargs['validators'].append(validate_latitude)
        super(LatitudeField, self).__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        """
        :returns: A :class:`~django.forms.FloatField` with ``max_value`` 90 and
            ``min_value`` -90.
        """
        kwargs.update({
            'max_value': 90,
            'min_value': -90,
        })
        return super(LatitudeField, self).formfield(**kwargs)


class LongitudeField(FloatField):
    """
    Bases: :class:`django.db.models.FloatField`

    All :ref:`default field options <django:common-model-field-options>`.

    The ``validators`` parameter will be appended with
    :func:`~validate_longitude` if not already present.
    """

    def __init__(self, *args, **kwargs):
        kwargs.setdefault('validators', [])
        if validate_longitude not in kwargs['validators']:
            kwargs['validators'].append(validate_longitude)
        super(LongitudeField, self).__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        """
        :returns: A :class:`~django.forms.FloatField` with ``max_value`` 180 and
            ``min_value`` -180.
        """
        kwargs.update({
            'max_value': 180,
            'min_value': -180,
        })
        return super(LongitudeField, self).formfield(**kwargs)


class OSMField(TextField):
    """
    Bases: :class:`django.db.models.TextField`

    :param str lat_field: The name of the latitude field. ``None`` (and thus
        standard behavior) by default.
    :param str lon_field: The name of the longitude field. ``None`` (and thus
        standard behavior) by default.

    All :ref:`default field options <django:common-model-field-options>`.
    """

    def __init__(self, *args, **kwargs):
        self._lat_field_name = kwargs.pop('lat_field', None)
        self._lon_field_name = kwargs.pop('lon_field', None)
        self.data_field_name = kwargs.pop('data_field', None)
        super(OSMField, self).__init__(*args, **kwargs)

    def contribute_to_class(self, cls, name):
        info_name = 'get_%s_info' % name
        if not hasattr(cls, info_name):
            def _func(obj):
                return Location(
                    getattr(obj, self.latitude_field_name),
                    getattr(obj, self.longitude_field_name),
                    getattr(obj, name),
                )
            setattr(cls, info_name, _func)

        super(OSMField, self).contribute_to_class(cls, name)

    def check(self, **kwargs):
        errors = super(OSMField, self).check(**kwargs)
        errors.extend(self._check_latitude_field())
        errors.extend(self._check_longitude_field())
        return errors

    def _check_latitude_field(self):
        opts = self.model._meta
        try:
            opts.get_field(self.latitude_field_name)
        except FieldDoesNotExist:
            return [
                checks.Error(
                    "The OSMField '%s' references the non-existent latitude field '%s'." % (
                        self.name, self.latitude_field_name,
                    ),
                    hint=None,
                    obj=self,
                    id='osm_field.E001',
                )
            ]
        else:
            return []

    def _check_longitude_field(self):
        opts = self.model._meta
        try:
            opts.get_field(self.longitude_field_name)
        except FieldDoesNotExist:
            return [
                checks.Error(
                    "The OSMField '%s' references the non-existent longitude field '%s'." % (
                        self.name, self.longitude_field_name,
                    ),
                    hint=None,
                    obj=self,
                    id='osm_field.E002',
                )
            ]
        else:
            return []

    def deconstruct(self):
        name, path, args, kwargs = super(OSMField, self).deconstruct()
        kwargs.update({
            'lat_field': self.latitude_field_name,
            'lon_field': self.longitude_field_name,
        })
        return name, path, args, kwargs

    def formfield(self, **kwargs):
        """
        :returns: A :class:`~osm_field.forms.OSMFormField` with a
            :class:`~osm_field.widgets.OSMWidget`.
        """
        widget_kwargs = {
            'lat_field': self.latitude_field_name,
            'lon_field': self.longitude_field_name,
        }

        if self.data_field_name:
            widget_kwargs['data_field'] = self.data_field_name

        defaults = {
            'form_class': OSMFormField,
            'widget': OSMWidget(**widget_kwargs),
        }
        defaults.update(kwargs)

        return super(OSMField, self).formfield(**defaults)

    @property
    def latitude_field_name(self):
        """
        The name of the related :class:`LatitudeField`.
        """
        if self._lat_field_name is None:
            self._lat_field_name = self.name + '_lat'
        return self._lat_field_name

    @property
    def longitude_field_name(self):
        """
        The name of the related :class:`LongitudeField`.
        """
        if self._lon_field_name is None:
            self._lon_field_name = self.name + '_lon'
        return self._lon_field_name



# -*- coding: utf-8 -*-
from django.core.exceptions import ValidationError


def validate_latitude(value):
    """
    Validates that the given value does not exceed the range [-90, 90].

    :raises: Raises a :class:`~django.core.exceptions.ValidationError` if
        ``value`` is not within the range.
    """
    if value < -90 or value > 90:
        raise ValidationError('invalid latitude')


def validate_longitude(value):
    """
    Validates that the given value does not exceed the range [-180, 180].

    :raises: Raises a :class:`~django.core.exceptions.ValidationError` if
        ``value`` is not within the range.
    """
    if value < -180 or value > 180:
        raise ValidationError('invalid longitude')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.conf import settings
from django.forms.widgets import Media, TextInput
from django.utils.html import format_html


def _get_js(debug=False):
    base = ['js/vendor/leaflet.js']
    if debug:
        base.extend(['js/osm_field.js'])
    else:
        base.extend(['js/osm_field.min.js'])
    return base


def _get_css(debug=False):
    base = ['css/vendor/leaflet.css']
    if debug:
        base.extend(['css/osm_field.css'])
    else:
        base.extend(['css/osm_field.min.css'])
    return base


class OSMWidget(TextInput):
    """
    Adds a OpenStreetMap Leaflet dropdown map to the front-end once the user
    focuses the form field. See :ref:`the usage chapter <usage-template-layer>`
    on how to integrate the CSS and JavaScript code.
    """

    @property
    def media(self):
        return Media(
            css={'screen': _get_css(settings.DEBUG)},
            js=_get_js(settings.DEBUG)
        )

    def __init__(self, lat_field, lon_field, data_field=None, attrs=None):
        attrs = {} if attrs is None else attrs.copy()
        attrs.update({
            'data-lat-field': lat_field,
            'data-lon-field': lon_field,
        })
        if data_field:
            attrs['data-data-field'] = data_field
        if 'class' in attrs:
            attrs['class'] += ' osmfield'
        else:
            attrs['class'] = 'osmfield'
        super(OSMWidget, self).__init__(attrs=attrs)

    def render(self, name, value, attrs=None, renderer=None):
        attrs = {} if attrs is None else attrs.copy()
        # For Django < 1.9, we need to grab self.attrs instead
        prefix = attrs.get('prefix', self.attrs.get('prefix', ''))
        if prefix:
            attrs.update({
                'data-lat-field': '{}-{}'.format(prefix, attrs.get('data-lat-field', self.attrs['data-lat-field'])),
                'data-lon-field': '{}-{}'.format(prefix, attrs.get('data-lon-field', self.attrs['data-lon-field'])),
            })
            if 'data-data-field' in self.attrs:
                attrs['data-data-field'] = '{}-{}'.format(prefix, attrs.get('data-data-field', self.attrs['data-data-field']))
        ret = super(OSMWidget, self).render(name, value, attrs=attrs)
        id_ = attrs['id']
        ret += self.render_osmfield(id_)
        return ret

    def render_osmfield(self, id_):
        # we need {{ and }} because of .format() working with {}
        return format_html('<script type="application/javascript">$(function()'
                           '{{$("#{0}").osmfield();}});</script>', id_)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from setuptools import setup

readme = open('README.rst').read()
history = open('HISTORY.rst').read().replace('.. :changelog:', '')


setup(
    name='django-osm-field',
    version='0.4.0',
    description='Django OpenStreetMap Field',
    long_description=readme + '\n\n' + history,
    author='Markus Holtermann, et al',
    author_email='info@markusholtermann.eu',
    url='https://github.com/MarkusH/django-osm-field',
    packages=[
        'osm_field',
    ],
    package_dir={'osm_field': 'osm_field'},
    include_package_data=True,
    install_requires=[
        'Django>=2.2',
        'six',
    ],
    license="MIT",
    zip_safe=False,
    keywords='OpenStreetMap, OSM, Django, Geo, Geoposition',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 2.2',
        'Framework :: Django :: 3.0',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
)
